// GENERATED FILE - DO NOT EDIT
function asp_4_init( formName, aspectId, unselectedText, debugFlag ) {
  asp_4 = new SelectSet( formName, aspectId, unselectedText, debugFlag );

  asp_menu_004_004 = new Menu(41,asp_4,1);
  asp_menu_004_004.addItem( 140, 0, true,  '004_003_001', '331', '331 - Histogrammes (none)' );
  asp_menu_004_004.addItem( 141, 2, false, '004_003_002', '332', '332 - Histogrammes (f)' );
  asp_menu_004_004.addItem( 142, 2, true,  '004_003_003', '333', '333 - Histogrammes (f->t)' );

  asp_menu_004 = new Menu(4,asp_4,0);
  asp_menu_004.addItem( 105, 2, true,  '004_001', '31', '31 - Chainage (f->t)' );
  asp_menu_004.addItem( 108, 2, false, '004_002', '32', '32 - Extension (f)' );
  asp_menu_004.addItem( 109, 0, false, '004_003', '33', '33 - Resource Histogrammes (drill down)',asp_menu_004_004 );
  asp_menu_004.addItem( 110, 0, false, '004_004', '34', '34 - Cash Flow (none)' );
  asp_menu_004.addItem( 111, 0, false, '004_005', '35', '35 - Progress Reports (none)' );
  asp_4.init()
}
